from hcloud.images.domain import Image
from hcloud.server_types.domain import ServerType
from hcloud.networks.domain import NetworkSubnet
from hcloud.locations.domain import Location
from hcloud import Client
client = Client(
    token="KccUEiddxtzGoLWSNC3V8tylq7MYHCjdnShtgasQ8jSbHqCjGoaa6Rq7yoz4uS23"
)
PREFIX = "478815"
YOUR_LOCAL_SSH_PUBKEY = "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAICZpOz3Rrdlg0daWESWGqSvMNSCrlROYU/Yy5/f+cPbh comment"

vnet_name = f"{PREFIX}-network"
vnet = client.networks.get_by_name(vnet_name) or None
if not vnet:
    vnet = client.networks.create(
        name=vnet_name, 
        ip_range="10.10.10.0/24", 
        subnets=[
            NetworkSubnet(ip_range="10.10.10.0/24", network_zone="eu-central", type="cloud")
        ]
    )

print(f"Utworzono sieć wirtualną {vnet.data_model.name} ({vnet.data_model.ip_range})")

ssh_name = f"{PREFIX}-ssh-key"
ssh_key = client.ssh_keys.get_by_name(ssh_name) or None
if not ssh_key:
    ssh_key = client.ssh_keys.create(name=ssh_name, public_key=YOUR_LOCAL_SSH_PUBKEY)

print(f"Klucz {ssh_key.data_model.name} został dodany: {ssh_key.data_model.public_key}")

cloud_init_db=r'''
packages:
  - apt-transport-https
  - ca-certificates
  - curl
  - gnupg-agent
  - software-properties-common
write_files:
  - path: /root/docker-compose.yml
    content: |
        version: '3.9'
        services:
          db:
              image: mysql:5.7
              restart: always
              ports:
                - "10.10.10.2:3306:3306"
              environment:
                MYSQL_ROOT_PASSWORD: veryStrongPassword
                MYSQL_DATABASE: gitea
                MYSQL_USER: gitea
                MYSQL_PASSWORD: gitea
              volumes:
                - db_data:/var/lib/mysql
          phpmyadmin:
              image: phpmyadmin
              restart: always
              ports:
                - "8080:80"
        volumes:
          db_data: {}
runcmd:
  - curl -fsSL https://download.docker.com/linux/ubuntu/gpg | apt-key add -
  - add-apt-repository "deb [arch=amd64] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable"
  - apt-get update -y
  - apt-get install -y docker-ce docker-ce-cli containerd.io
  - curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/local/bin/docker-compose
  - chmod +x /usr/local/bin/docker-compose
  - systemctl start docker
  - systemctl enable docker
  - cd /root/ && docker-compose up -d
'''

db_server = client.servers.create(
    name=f"{PREFIX}-db", 
    server_type=ServerType("cx11"), 
    image=Image(name="ubuntu-20.04"), 
    ssh_keys=[ssh_key], 
    networks=[vnet], 
    location=Location("hel1"), 
    user_data=cloud_init_db
)

db_server.action.wait_until_finished()
print(f"Tworzenie serwera db: {db_server.action.complete}")

db_server = client.servers.get_by_name(f"{PREFIX}-db")
print(f"Serwer: {db_server.data_model.name}\n\tpubliczne IP: {db_server.data_model.public_net.ipv4.ip}\n\tprywatne IP: {db_server.data_model.private_net[0].ip}")

cloud_init_git=r'''
packages:
  - apt-transport-https
  - ca-certificates
  - curl
  - gnupg-agent
  - software-properties-common
write_files:
  - path: /root/docker-compose.yml
    content: |
        version: '3.9'
        services:
          server:
            image: gitea/gitea:1.15.6-rootless
            environment:
              GITEA_database_DB_TYPE: mysql
              GITEA_database_HOST: 10.10.10.2:3306
              GITEA_database_NAME: gitea
              GITEA_database_USER: gitea
              GITEA_database_PASSWD: veryStrongPassword
            restart: always
            volumes:
              - ./data:/root/gitea
              - ./config:/root/gitea/config
              - /etc/timezone:/etc/timezone:ro
              - /etc/localtime:/etc/localtime:ro
              - /mnt/volume:/data
            ports:
              - "3000:3000"
              - "222:22"
runcmd:
  - curl -fsSL https://download.docker.com/linux/ubuntu/gpg | apt-key add -
  - add-apt-repository "deb [arch=amd64] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable"
  - apt-get update -y
  - apt-get install -y docker-ce docker-ce-cli containerd.io
  - curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/local/bin/docker-compose
  - chmod +x /usr/local/bin/docker-compose
  - systemctl start docker
  - systemctl enable docker
  - cd /root/ && docker-compose up -d
'''

git_server = client.servers.create(
    name=f"{PREFIX}-gitea", 
    server_type=ServerType("cx11"), 
    image=Image(name="ubuntu-20.04"), 
    ssh_keys=[ssh_key], 
    networks=[vnet], 
    location=Location("hel1"), 
    user_data=cloud_init_git
)

git_server.action.wait_until_finished()
print(f"Tworzenie serwera gitea: {git_server.action.complete}")

git_server = client.servers.get_by_name(f"{PREFIX}-gitea")
print(f"Serwer: {git_server.data_model.name}\n\tpubliczne IP: {git_server.data_model.public_net.ipv4.ip}\n\tprywatne IP: {git_server.data_model.private_net[0].ip}")
